<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads.
 *
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2022 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 *  If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Core\Html\Callbacks;

use _JchOptimizeVendor\GuzzleHttp\Psr7\Uri;
use JchOptimize\Core\Helper;
use JchOptimize\Core\Html\FilesManager;
use JchOptimize\Core\Html\Processor as HtmlProcessor;
use JchOptimize\Core\Http2Preload;
use JchOptimize\Core\Uri\UriNormalizer;
use JchOptimize\Platform\Excludes;
use JchOptimize\Platform\Profiler;
use Joomla\Registry\Registry;

\defined('_JCH_EXEC') or exit('Restricted access');
class CombineJsCss extends \JchOptimize\Core\Html\Callbacks\AbstractCallback
{
    /**
     * @var array Array of excludes parameters
     */
    private array $excludes;

    /**
     * @var string Section of the HTML being processed
     */
    private string $section = 'head';

    private FilesManager $filesManager;

    private Http2Preload $http2Preload;

    private HtmlProcessor $htmlProcessor;

    /**
     * CombineJsCss constructor.
     */
    public function __construct(Registry $params, FilesManager $filesManager, Http2Preload $http2Preload, HtmlProcessor $htmlProcessor)
    {
        parent::__construct($params);
        $this->filesManager = $filesManager;
        $this->http2Preload = $http2Preload;
        $this->htmlProcessor = $htmlProcessor;
        $this->setupExcludes();
    }

    /**
     * Callback function used to remove urls of css and js files in head tags.
     *
     * @param array $matches Array of all matches
     *
     * @return string Returns the url if excluded, empty string otherwise
     */
    public function processMatches($matches): string
    {
        if (empty($matches[0])) {
            return $matches[0];
        }
        if (isset($matches[4])) {
            $uri = $matches['url'] = UriNormalizer::normalize(new Uri(\trim($matches[4])));
        } else {
            $matches['content'] = $matches[2];
        }
        if (\preg_match('#^<!--#', $matches[0])) {
            return $matches[0];
        }
        // If url is invalid just remove it, sometimes they cause the page to download again so most likely
        // would be better
        if (isset($matches['url']) && Helper::uriInvalid($uri)) {
            return '';
        }
        $type = 0 == \strcasecmp($matches[1], 'script') ? 'js' : 'css';
        // Remove js files
        if ('js' == $type && isset($matches['url']) && Helper::findExcludes(@$this->excludes[$this->section]['remove']['js'], $uri)) {
            return '';
        }
        // Remove css files
        if ('css' == $type && isset($matches['url']) && Helper::findExcludes(@$this->excludes[$this->section]['remove']['css'], $uri)) {
            return '';
        }
        if ('js' == $type && (!$this->params->get('javascript', '1') || !$this->params->get('combine_files_enable', '1') || $this->htmlProcessor->isAmpPage)) {
            $deferred = $this->filesManager->isFileDeferred($matches[0]);
            if (isset($matches['url'])) {
                $this->http2Preload->add($uri, 'script', $deferred);
            }

            return $matches[0];
        }
        if ('css' == $type && (!$this->params->get('css', '1') || !$this->params->get('combine_files_enable', '1') || $this->htmlProcessor->isAmpPage)) {
            if (isset($matches['url'])) {
                $this->http2Preload->add($uri, 'style');
            }

            return $matches[0];
        }
        $this->filesManager->setExcludes($this->excludes[$this->section]);

        return $this->filesManager->processFiles($type, $matches);
    }

    public function setSection($section)
    {
        $this->section = $section;
    }

    /**
     * Retrieves all exclusion parameters for the Combine Files feature.
     */
    private function setupExcludes()
    {
        JCH_DEBUG ? Profiler::start('SetUpExcludes') : null;
        $this->excludes = [];
        $aExcludes = [];
        $oParams = $this->params;
        // These parameters will be excluded while preserving execution order
        $aExJsComp = $this->getExComp($oParams->get('excludeJsComponents_peo', ''));
        $aExCssComp = $this->getExComp($oParams->get('excludeCssComponents', ''));
        $aExcludeJs_peo = Helper::getArray($oParams->get('excludeJs_peo', ''));
        $aExcludeCss_peo = Helper::getArray($oParams->get('excludeCss', ''));
        $aExcludeScript_peo = Helper::getArray($oParams->get('excludeScripts_peo', ''));
        $aExcludeStyle_peo = Helper::getArray($oParams->get('excludeStyles', ''));
        $aExcludeScript_peo = \array_map(function ($script) {
            $script['script'] = \stripslashes($script['script']);

            return $script;
        }, $aExcludeScript_peo);
        $this->excludes['excludes_peo']['js'] = \array_merge($aExcludeJs_peo, $aExJsComp, [['url' => '.com/maps/api/js'], ['url' => '.com/jsapi'], ['url' => '.com/uds'], ['url' => 'typekit.net'], ['url' => 'cdn.ampproject.org'], ['url' => 'googleadservices.com/pagead/conversion']], Excludes::head('js'));
        $this->excludes['excludes_peo']['css'] = \array_merge($aExcludeCss_peo, $aExCssComp, Excludes::head('css'));
        $this->excludes['excludes_peo']['js_script'] = $aExcludeScript_peo;
        $this->excludes['excludes_peo']['css_script'] = $aExcludeStyle_peo;
        $this->excludes['critical_js']['js'] = Helper::getArray($oParams->get('pro_criticalJs', ''));
        $this->excludes['critical_js']['script'] = Helper::getArray($oParams->get('pro_criticalScripts', ''));
        $this->excludes['remove']['js'] = Helper::getArray($oParams->get('remove_js', ''));
        $this->excludes['remove']['css'] = Helper::getArray($oParams->get('remove_css', ''));
        $aExcludes['head'] = $this->excludes;
        if (1 == $this->params->get('bottom_js', '0')) {
            $this->excludes['excludes_peo']['js_script'] = \array_merge($this->excludes['excludes_peo']['js_script'], [['script' => 'var google_conversion'], ['script' => '.write(', 'dontmove' => 'on']], Excludes::body('js', 'script'));
            $this->excludes['excludes_peo']['js'] = \array_merge($this->excludes['excludes_peo']['js'], [['url' => '.com/recaptcha/api']], Excludes::body('js'));
            $aExcludes['body'] = $this->excludes;
        }
        JCH_DEBUG ? Profiler::stop('SetUpExcludes', \true) : null;
        $this->excludes = $aExcludes;
    }

    /**
     * Generates regex for excluding components set in plugin params.
     */
    private function getExComp($excludedComParams): array
    {
        $components = Helper::getArray($excludedComParams);
        $excludedComponents = [];
        if (!empty($components)) {
            $excludedComponents = \array_map(function ($value) {
                if (isset($value['url'])) {
                    $value['url'] = \rtrim($value['url'], '/').'/';
                } else {
                    $value = \rtrim($value, '/').'/';
                }

                return $value;
            }, $components);
        }

        return $excludedComponents;
    }
}
